@@ -31,16 +31,18 @@ along with this program.  If not, see <http://www.gnu.org/licenses/>.
 
 // KWin
 #include "atoms.h"
-#include "client.h"
+#include <client.h>
 #include "cursor.h"
 #include "input.h"
 #include "main.h"
 #include "screens.h"
 #include "utils.h"
-#include "workspace.h"
+#include <workspace.h>
 #include "virtualdesktops.h"
 // DBus generated
 #include "screenlocker_interface.h"
+// frameworks
+#include <KConfigGroup>
 // Qt
 #include <QSharedPointer>
 #include <QTimer>
@@ -48,6 +50,7 @@ along with this program.  If not, see <http://www.gnu.org/licenses/>.
 #include <QTextStream>
 #include <QtDBus/QDBusInterface>
 #include <QtDBus/QDBusPendingCall>
+#include <QWidget>
 
 namespace KWin {
 
@@ -281,12 +284,14 @@ void Edge::switchDesktop(const QPoint &cursorPos)
         if (desktop != interimDesktop)
             pos.setY(OFFSET);
     }
+#ifndef KWIN_UNIT_TEST
     if (Client *c = Workspace::self()->getMovingClient()) {
         if (c->rules()->checkDesktop(desktop) != int(desktop)) {
             // user attempts to move a client to another desktop where it is ruleforced to not be
             return;
         }
     }
+#endif
     vds->setCurrent(desktop);
     if (vds->current() != oldDesktop) {
         m_pushBackBlocked = true;
@@ -867,23 +872,27 @@ void ScreenEdges::recreateEdges()
     QList<Edge*> oldEdges(m_edges);
     m_edges.clear();
     const QRect fullArea(0, 0, displayWidth(), displayHeight());
+    QRegion processedRegion;
     for (int i=0; i<screens()->count(); ++i) {
-        const QRect screen = screens()->geometry(i);
-        if (isLeftScreen(screen, fullArea)) {
-            // left most screen
-            createVerticalEdge(ElectricLeft, screen, fullArea);
-        }
-        if (isRightScreen(screen, fullArea)) {
-            // right most screen
-            createVerticalEdge(ElectricRight, screen, fullArea);
-        }
-        if (isTopScreen(screen, fullArea)) {
-            // top most screen
-            createHorizontalEdge(ElectricTop, screen, fullArea);
-        }
-        if (isBottomScreen(screen, fullArea)) {
-            // bottom most screen
-            createHorizontalEdge(ElectricBottom, screen, fullArea);
+        const QRegion screen = QRegion(screens()->geometry(i)).subtracted(processedRegion);
+        processedRegion += screen;
+        Q_FOREACH (const QRect &screenPart, screen.rects()) {
+            if (isLeftScreen(screenPart, fullArea)) {
+                // left most screen
+                createVerticalEdge(ElectricLeft, screenPart, fullArea);
+            }
+            if (isRightScreen(screenPart, fullArea)) {
+                // right most screen
+                createVerticalEdge(ElectricRight, screenPart, fullArea);
+            }
+            if (isTopScreen(screenPart, fullArea)) {
+                // top most screen
+                createHorizontalEdge(ElectricTop, screenPart, fullArea);
+            }
+            if (isBottomScreen(screenPart, fullArea)) {
+                // bottom most screen
+                createHorizontalEdge(ElectricBottom, screenPart, fullArea);
+            }
         }
     }
     // copy over the effect/script reservations from the old edges
