@@ -20,6 +20,7 @@ along with this program.  If not, see <http://www.gnu.org/licenses/>.
 
 // own
 #include "tabboxhandler.h"
+#include <config-kwin.h>
 #include <kwinglobals.h>
 #include "xcbutils.h"
 // tabbox
@@ -30,9 +31,7 @@ along with this program.  If not, see <http://www.gnu.org/licenses/>.
 #include "scripting/scripting.h"
 #include "switcheritem.h"
 // Qt
-#include <QApplication>
 #include <QDebug>
-#include <QDesktopWidget>
 #include <QKeyEvent>
 #include <QModelIndex>
 #include <QStandardPaths>
@@ -231,36 +230,43 @@ void TabBoxHandlerPrivate::endHighlightWindows(bool abort)
 #ifndef KWIN_UNIT_TEST
 QObject *TabBoxHandlerPrivate::createSwitcherItem(bool desktopMode)
 {
-    auto findSwitcher = [this, desktopMode] {
-        QString constraint = QStringLiteral("[X-KDE-PluginInfo-Name] == '%1'").arg(config.layoutName());
-        const QString type = desktopMode ? QStringLiteral("KWin/DesktopSwitcher") : QStringLiteral("KWin/WindowSwitcher");
-        KService::List offers = KServiceTypeTrader::self()->query(type, constraint);
-        if (offers.isEmpty()) {
-            // load default
-            constraint = QStringLiteral("[X-KDE-PluginInfo-Name] == '%1'").arg(QStringLiteral("informative"));
-            offers = KServiceTypeTrader::self()->query(type, constraint);
+    // first try look'n'feel package
+    QString file = QStandardPaths::locate(QStandardPaths::GenericDataLocation,
+                                          QStringLiteral("plasma/look-and-feel/%1/contents/%2")
+                                              .arg(config.layoutName())
+                                              .arg(desktopMode ? QStringLiteral("desktopswitcher/DesktopSwitcher.qml") : QStringLiteral("windowswitcher/WindowSwitcher.qml")));
+    if (file.isNull()) {
+        auto findSwitcher = [this, desktopMode] {
+            QString constraint = QStringLiteral("[X-KDE-PluginInfo-Name] == '%1'").arg(config.layoutName());
+            const QString type = desktopMode ? QStringLiteral("KWin/DesktopSwitcher") : QStringLiteral("KWin/WindowSwitcher");
+            KService::List offers = KServiceTypeTrader::self()->query(type, constraint);
             if (offers.isEmpty()) {
-                qDebug() << "could not find default window switcher layout";
-                return KService::Ptr();
+                // load default
+                constraint = QStringLiteral("[X-KDE-PluginInfo-Name] == '%1'").arg(QStringLiteral("informative"));
+                offers = KServiceTypeTrader::self()->query(type, constraint);
+                if (offers.isEmpty()) {
+                    qDebug() << "could not find default window switcher layout";
+                    return KService::Ptr();
+                }
             }
+            return offers.first();
+        };
+        KService::Ptr service = findSwitcher();
+        if (!service) {
+            return nullptr;
         }
-        return offers.first();
-    };
-    KService::Ptr service = findSwitcher();
-    if (!service) {
-        return nullptr;
+        if (service->property(QStringLiteral("X-Plasma-API")).toString() != QStringLiteral("declarativeappletscript")) {
+            qDebug() << "Window Switcher Layout is no declarativeappletscript";
+            return nullptr;
+        }
+        auto findScriptFile = [desktopMode, service] {
+            const QString pluginName = service->property(QStringLiteral("X-KDE-PluginInfo-Name")).toString();
+            const QString scriptName = service->property(QStringLiteral("X-Plasma-MainScript")).toString();
+            const QString type = desktopMode ? QStringLiteral("/desktoptabbox/") : QStringLiteral("/tabbox/");
+            return QStandardPaths::locate(QStandardPaths::GenericDataLocation, QStringLiteral(KWIN_NAME) + type + pluginName + QStringLiteral("/contents/") + scriptName);
+        };
+        file = findScriptFile();
     }
-    if (service->property(QStringLiteral("X-Plasma-API")).toString() != QStringLiteral("declarativeappletscript")) {
-        qDebug() << "Window Switcher Layout is no declarativeappletscript";
-        return nullptr;
-    }
-    auto findScriptFile = [desktopMode, service] {
-        const QString pluginName = service->property(QStringLiteral("X-KDE-PluginInfo-Name")).toString();
-        const QString scriptName = service->property(QStringLiteral("X-Plasma-MainScript")).toString();
-        const QString type = desktopMode ? QStringLiteral("/desktoptabbox/") : QStringLiteral("/tabbox/");
-        return QStandardPaths::locate(QStandardPaths::GenericDataLocation, QStringLiteral(KWIN_NAME) + type + pluginName + QStringLiteral("/contents/") + scriptName);
-    };
-    const QString file = findScriptFile();
     if (file.isNull()) {
         qDebug() << "Could not find QML file for window switcher";
         return nullptr;
